//+------------------------------------------------------------------+
//|                                       MagicStar_OrderInclude.mqh |
//|                                             2009.06.19 SwingMan |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright " 2009.06.19 SwingMan"
#property link      ""
//-- adapted for MagicGirl

/* PROCEDURES:
void Get_ActivOrder()
void Get_LotsNumber()
void SendOrder_BuyMarket(double nLots, int iMagicNumber, string stComment)
void SendOrder_SellMarket(double nLots, int iMagicNumber, string stComment)
bool Modify_LongStopLoss(int iMagicNumber, double NewStopLoss)
bool Modify_ShortStopLoss(int iMagicNumber, double NewStopLoss)
void Send_ErrorDescription(string sSymbol, string sText)
bool SendOrders_IsAllowed()
*/

#include <stdlib.mqh>

double Slippage_Entry = 5;
double Slippage_Exit  = 10;

color CLR_Buy = Blue;
color CLR_Sell = Red;
color CLR_Exit = Gold;

//---- variables trading
int nActivOrders, ActivOrderTyp;
double OpenValueOrder, StopLossOrder;
datetime OpenTimeOrder;
double ProfitOrder, NumberLotsOrder;
double nLots;
//---- trading orders
double StopLossTrade;
double InitialStopLoss, BreakEvenStop, StrongStopLoss;
int mySystemSignal  = 1; //-- long trend; avoid reentrys in trend

//####################################################################
//+------------------------------------------------------------------+
//|   Get Activ Orders
//+------------------------------------------------------------------+
void Get_ActivOrder(int iMagicNumber)
{  
   int total = OrdersTotal();   

   string sSymbol    = Symbol();
   nActivOrders  = 0;
   ProfitOrder   = 0;
   ActivOrderTyp = -1;

   //---- Order 1
   for(int i=0; i<total; i++) 
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {      
         if (OrderSymbol()      == sSymbol && 
             OrderCloseTime()   == 0 &&
             OrderMagicNumber() == iMagicNumber)
         {
            OpenValueOrder = OrderOpenPrice();
            StopLossOrder  = OrderStopLoss();
            ProfitOrder    = OrderProfit();
            OpenTimeOrder  = OrderOpenTime();
            NumberLotsOrder= OrderLots();
            nActivOrders++;                
            ActivOrderTyp = OrderType();
               
            if (mySystemSignal == 0) {
               if (OrderType() == OP_BUY)
                  mySystemSignal =  1; else
               if (OrderType() == OP_SELL)
                  mySystemSignal = -1;
            }   
            break;
         }
      }      
   }  
   return;
}


//####################################################################
//####################################################################
//+------------------------------------------------------------------+
//    Send Order BuyMarket
//+------------------------------------------------------------------+
void SendOrder_BuyMarket(string sSymbol, double nLots, int iMagicNumber, string stComment)
{
   double dPrice = MarketInfo(sSymbol, MODE_ASK);
   string sComment = sSymbol + " -buy- " + stComment;
   if (SendOrders_IsAllowed() == true) {
      int ticket1=OrderSend(sSymbol,OP_BUY,nLots,dPrice,Slippage_Entry,0,0, sComment, iMagicNumber,0,CLR_Buy);      
   }
   if(ticket1<0) 
      Send_ErrorDescription(sSymbol, " BUY " + stComment);
   else
   {
      StopLossTrade   = 0;
      InitialStopLoss = 0;
      BreakEvenStop   = 0;
      StrongStopLoss  = 0;
      mySystemSignal  = 1; //-- long trend; avoid reentrys in trend
   }   
   return;
}

//+------------------------------------------------------------------+
//    Send Order SellMarket
//+------------------------------------------------------------------+
void SendOrder_SellMarket(string sSymbol, double nLots, int iMagicNumber, string stComment)
{
   double dPrice = MarketInfo(sSymbol, MODE_BID);
   string sComment = sSymbol + " -sell- " + stComment;
   if (SendOrders_IsAllowed() == true) {
      int ticket1 = OrderSend(sSymbol,OP_SELL,nLots,dPrice,Slippage_Entry,0,0, sComment, iMagicNumber,0,CLR_Sell);
   }
   if(ticket1<0)
      Send_ErrorDescription(sSymbol, " SELL " + stComment); 
   else
   {
      StopLossTrade   = 0;
      InitialStopLoss = 0;
      BreakEvenStop   = 0;
      StrongStopLoss  = 0;
      mySystemSignal  = -1; //-- short trend; avoid reentrys in trend   
   }   
   return;   
}


//+------------------------------------------------------------------+
//    Send Orders Is Allowed
//+------------------------------------------------------------------+
bool SendOrders_IsAllowed()
{
return(true);

   bool SendOrderIsOK = false;
   while (SendOrderIsOK == true) {      
      if (IsTradeAllowed()     == true && 
          IsTradeContextBusy() == false) // &&
          //RefreshRates()       == true) 
         SendOrderIsOK = true;
      else
         Sleep(300);   
   }
   return(SendOrderIsOK);
}

//####################################################################
//####################################################################
//+------------------------------------------------------------------+
//| Close Activ Order
//+------------------------------------------------------------------+
bool Close_ActivOrder(string sSymbol, int iMagicNumber)
{
   bool CloseResult;  

   int total=OrdersTotal();

   for(int i=0; i<total; i++) {
      if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
         if (OrderSymbol()      == sSymbol && 
             OrderCloseTime()   == 0 &&
             OrderMagicNumber() == iMagicNumber) {
               
            if (OrderType() == OP_BUY) {
               if (SendOrders_IsAllowed() == true) {
                  double dBid = MarketInfo(sSymbol, MODE_BID);
                  CloseResult = OrderClose(OrderTicket(), OrderLots(), dBid, Slippage_Exit, CLR_Exit);
               }   
               if (CloseResult == false) Send_ErrorDescription(OrderSymbol(), " Buy-Exit " + dBid);
               break;
            } else
               
            if (OrderType() == OP_SELL) {
               if (SendOrders_IsAllowed() == true) {
                  double dAsk = MarketInfo(sSymbol, MODE_ASK);
                  CloseResult = OrderClose(OrderTicket(), OrderLots(), dAsk, Slippage_Exit, CLR_Exit);
               }   
               if (CloseResult == false) Send_ErrorDescription(OrderSymbol(), " Sell-Exit " + dAsk);
               break;
            }   
         }
      }
   }
   return(CloseResult);
}

//####################################################################
//+------------------------------------------------------------------+
//    Send_ErrorDescription
//+------------------------------------------------------------------+
void Send_ErrorDescription(string sSymbol, string sText)
{
   int err=GetLastError(); 
   if (sText != "")
   string st = sSymbol + " error(" + err + "): " + ErrorDescription(err)+ " -> " + sText; else
          st = sSymbol + " error(" + err + "): " + ErrorDescription(err);
   Alert(st);
   return;
}


//####################################################################
//+------------------------------------------------------------------+
//|   Get Lots Number
//+------------------------------------------------------------------+
double Get_LotsNumber(bool MoneyManagement, double Risk)
{
double Lots;
   //Money Management sequence
   if (MoneyManagement) {
      if (Risk<1 || Risk>100) {
         Comment("Invalid Risk Value.");
         Lots = Min_Lots;
         return(0);
      }
      else  {
         Lots=MathFloor((AccountFreeMargin()*AccountLeverage()*Risk*Point*BrokerMultiplier*100)/(Ask*MarketInfo(Symbol(),MODE_LOTSIZE)*MarketInfo(Symbol(),MODE_MINLOT)))*MarketInfo(Symbol(),MODE_MINLOT);
      }
   }
   else
      Lots = Min_Lots;
   
   if (Lots < Min_Lots) Lots = Min_Lots;   
   return(Lots);   
}



//####################################################################
//####################################################################
//####################################################################
//+------------------------------------------------------------------+
//|   Modify Long StopLoss and Target
//+------------------------------------------------------------------+
bool Modify_LongStopLossTarget(string sSymbol, int iMagicNumber, double NewStopLoss, double NewTarget)
{  
   bool modify = false; 
   int iDigits = MarketInfo(sSymbol,MODE_DIGITS);
   double dSpace = MarketInfo(sSymbol,MODE_STOPLEVEL)*MarketInfo(sSymbol,MODE_POINT);
   double dBid   = MarketInfo(sSymbol, MODE_BID);
   double dStopLoss = NormalizeDouble(NewStopLoss, iDigits);  
   double dTarget = NormalizeDouble(NewTarget, iDigits);
   
   if (dStopLoss >= dBid - dSpace) return(false);
   
   int total=OrdersTotal();
   
   for(int i=0; i<total; i++) {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
      
         if (OrderSymbol()      == sSymbol && 
             OrderType()        == OP_BUY &&
             OrderCloseTime()   == 0 && 
             OrderMagicNumber() == iMagicNumber) 
         {
            //while (modify == false) { 
               if (SendOrders_IsAllowed() == true)
                  modify = OrderModify(OrderTicket(), OrderOpenPrice(), dStopLoss, dTarget, 0, CLR_NONE);
               if (modify == false) {
                  string sModify = " Buy ";                  
                  string sPrice  = ", SL= " + DoubleToStr(dStopLoss, iDigits);
                  Send_ErrorDescription(sSymbol, " Modify SL" + sModify + sPrice);
               }               
            //}
            break;
         }
      }
   }
   return(modify);
}


//+------------------------------------------------------------------+
//|   Modify Short StopLoss and Target
//+------------------------------------------------------------------+
bool Modify_ShortStopLossTarget(string sSymbol, int iMagicNumber, double NewStopLoss, double NewTarget)
{
   bool modify = false; 
   int iDigits = MarketInfo(sSymbol,MODE_DIGITS);
   double dSpace = MarketInfo(sSymbol,MODE_STOPLEVEL)*MarketInfo(sSymbol,MODE_POINT);
   double dAsk   = MarketInfo(sSymbol, MODE_ASK);
   double dStopLoss = NormalizeDouble(NewStopLoss, iDigits);   
   double dTarget = NormalizeDouble(NewTarget, iDigits);
   
   if (dStopLoss <= dAsk + dSpace) return(false);
   
   int total=OrdersTotal();
   
   for(int i=0; i<total; i++) {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
      
         if (OrderSymbol()      == sSymbol && 
             OrderType()        == OP_SELL &&
             OrderCloseTime()   == 0 && 
             OrderMagicNumber() == iMagicNumber) 
         {   
            //while (modify == false) {           
               if (SendOrders_IsAllowed() == true)
                  modify = OrderModify(OrderTicket(), OrderOpenPrice(), dStopLoss, dTarget, 0, CLR_NONE);            
               if (modify == false) {
                  string sModify = " Sell";
                  string sPrice  = ", SL= " +DoubleToStr(dStopLoss, iDigits);
                  Send_ErrorDescription(sSymbol, " Modify SL" + sModify + sPrice);
               }
            //}     
            break;            
         }
      }
   }
   return(modify);
}

